module Evaluation.Function
  ( evalFunction
  ) where

import Prelude hiding (String, FilePath)
import qualified Prelude

import Types (Function(..), Value(..), String(..), Template(..), fromValue)

import Data.Char (toLower)
import qualified Data.Text as T

fileComponents :: Prelude.String -> (Prelude.String, Prelude.String)
fileComponents s =
  let (lastRev, firstRev) = span (/= '.') $ reverse s
  in case firstRev of
       _ : firstRev' -> (reverse firstRev', reverse lastRev)
       [] -> (reverse lastRev, "")

makeString :: Prelude.String -> Value
makeString = String . StringWrapper

unStringWrapper :: Value -> Prelude.String
unStringWrapper = \case
  String (StringWrapper s) -> s
  _ -> error "unexpected"

evalFunction :: Function -> Value -> Value
evalFunction f x = case (f, x) of
  (AppendStrings, Tuple (String (StringWrapper s0), String (StringWrapper s1))) ->
    makeString (s0 ++ s1)

  (ConcatStrings, List vs) ->
    makeString $ concatMap unStringWrapper vs

  (AppendTexts, Tuple (Text t0, Text t1)) ->
    Text $ T.append t0 t1

  (ConcatTexts, List vs) ->
    Text $ T.concat $ map fromValue vs

  (JoinPaths, Tuple (String (StringWrapper s0), String (StringWrapper s1))) ->
    makeString (s0 ++ "/" ++ s1)

  (FileComponents, String (StringWrapper s)) ->
    let (base, ext) = fileComponents s
    in Tuple (makeString base, makeString ext)

  (LowerString, String (StringWrapper s)) ->
    makeString $ map toLower s

  (ElemOf, Tuple (y, List ys)) ->
    Bool (y `elem` ys)

  (MakeTemplate, Tuple (Text t, Text c)) ->
    let (beforeContent, after) = T.breakOn c t
        afterContent = T.drop (T.length c) after
    in Template $ TemplateParts beforeContent afterContent

  (ApplyTemplate, Tuple (Template (TemplateParts beforeContent afterContent), Text t)) ->
    Text $ T.concat [beforeContent, t, afterContent]

  (ToText, String (StringWrapper s)) ->
    Text $ T.pack s

  _ ->
    error "unexpected combination of function and argument type"
