module Types.Value
  ( Value(..)
  , Valuable(..)
  ) where

import Prelude hiding (String)

import Types.Values

import Data.Text (Text)
import Language.Haskell.TH.Syntax (Lift)

-- Note: We use a wrapper for the String type in order to be able to define the
-- general Valuable [a] instance further down.  Otherwise it would conflict with
-- our Valuable String instance, since the non-wrapped String type is just an
-- alias for [Char].
data Value = String String
           | Text Text
           | Bool Bool
           | ImageConversionSettings ImageConversionSettings
           | Template Template
           | Empty
           | Tuple (Value, Value)
           | List [Value]
  deriving (Eq, Show, Lift)

class Valuable a where
  toValue :: a -> Value
  fromValue :: Value -> a

instance Valuable String where
  toValue = String
  fromValue = \case
    String a -> a
    _ -> error "unexpected"

instance Valuable Text where
  toValue = Text
  fromValue = \case
    Text a -> a
    _ -> error "unexpected"

instance Valuable Bool where
  toValue = Bool
  fromValue = \case
    Bool a -> a
    _ -> error "unexpected"

instance Valuable ImageConversionSettings where
  toValue = ImageConversionSettings
  fromValue = \case
    ImageConversionSettings a -> a
    _ -> error "unexpected"

instance Valuable a => Valuable [a] where
  toValue = List . map toValue
  fromValue = \case
    List a -> map fromValue a
    _ -> error "unexpected"
